import sys

program = [line.strip().split() for line in sys.stdin if line.strip() != ""]


def test(input: int) -> int:
    vars = {"input": input}

    labels = {}
    for i, line in enumerate(program):
        match line:
            case ["LABEL", name]:
                labels[name] = i

    pc = 0

    def getvar(name):
        if name in vars:
            return vars[name]
        raise Exception(f"no such variable {name} (at line {pc})")

    while True:
        match program[pc]:
            case ["SET", name, value]:
                vars[name] = int(value)
            case ["LABEL", name]:
                pass
            case ["JUMP", name]:
                pc = labels[name]
                continue
            case ["JUMP_IF", cond, label, var1, var2]:
                val1, val2 = getvar(var1), getvar(var2)
                match cond.removeprefix("!"):
                    case "EQ":
                        should_jump = val1 == val2
                    case "LT":
                        should_jump = val1 < val2
                    case "LTE":
                        should_jump = val1 <= val2
                    case "GT":
                        should_jump = val1 > val2
                    case "GTE":
                        should_jump = val1 >= val2
                    case _:
                        raise Exception(f"unknown cond {cond} (at line {pc})")
                if cond.startswith("!"):
                    should_jump = not should_jump
                if should_jump:
                    pc = labels[label]
                    continue
            case ["GETBIT", var1, var2, var3]:
                vars[var3] = (getvar(var1) >> getvar(var2)) & 1
            case ["SETBIT", var1, var2, var3]:
                if var3 not in vars:
                    vars[var3] = 0
                else:
                    vars[var3] &= ~(1 << getvar(var2))
                vars[var3] |= max(getvar(var1), 1) << getvar(var2)
            case ["RETURN", name]:
                return getvar(name)
            case _:
                raise Exception(f"unexpected token at line {pc}")
        pc += 1


def is_zoobly(n):
    if n >= 2**9:
        return False
    bits = format(n, "09b")
    matrix = [[bits[i * 3 + j] for j in range(3)] for i in range(3)]
    transpose = [[matrix[j][i] for j in range(3)] for i in range(3)]
    return matrix == transpose


for i in range(512):
    expect = is_zoobly(i)
    got = test(i)
    if expect != got:
        print(f"{i}: expected {expect}, got {got}")
